==> libosl_0.3.0.orig.tar.gz: libosl_0.3.0.orig/test/eval/endgame/attackDefense.t.cc <==
#include "osl/eval/endgame/attackDefense.h"
#include "osl/eval/evalTraits.h"
#include "osl/eval/pieceEval.h"
#include "osl/record/csaString.h"
#include "../consistencyTest.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace osl;
using namespace osl::eval;
using namespace osl::eval::endgame;

class EndgameAttackDeffenseTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(EndgameAttackDeffenseTest);
  CPPUNIT_TEST(testStand);
  CPPUNIT_TEST(testMax);
  CPPUNIT_TEST(testConsistentUpdate);
  CPPUNIT_TEST(testConsistentExpect);
  CPPUNIT_TEST(testYagura);
  CPPUNIT_TEST(testBlockingSilver);
  CPPUNIT_TEST(testInitial);
  CPPUNIT_TEST_SUITE_END();
public:
  void testMax();
  void testStand();
  void testConsistentUpdate();
  void testConsistentExpect();
  void testYagura();
  void testBlockingSilver();
  void testInitial();
};

CPPUNIT_TEST_SUITE_REGISTRATION(EndgameAttackDeffenseTest);

void EndgameAttackDeffenseTest::testInitial()
{
  const NumEffectState state(CsaString(
			       "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			       "P2 * -HI *  *  *  *  * -KA * \n"
			       "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
			       "P4 *  *  *  *  *  *  *  *  * \n"
			       "P5 *  *  *  *  *  *  *  *  * \n"
			       "P6 *  *  *  *  *  *  *  *  * \n"
			       "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
			       "P8 * +KA *  *  *  *  * +HI * \n"
			       "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			       "+\n").getInitialState());
  AttackDefense eval(state);
  CPPUNIT_ASSERT_EQUAL(0, eval.value());
}

void EndgameAttackDeffenseTest::testMax() 
{
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  std::string file_name;
  for (int i=0; i<(isShortTest ? 10 : 100) && (ifs >> file_name) ; i++)
  {
    if ((i % 20) == 0)
      std::cerr << '.';
    if (file_name == "") 
      break;
    file_name = OslConfig::testCsaFile(file_name);

    const Record record=CsaFile(file_name).getRecord();
    const vector<osl::Move> moves=record.getMoves();

    NumEffectState state(record.getInitialState());
    for (unsigned int i=0; i<moves.size(); i++)
    {
      const Move m = moves[i];
      ApplyMoveOfTurn::doMove(state, m);

      for (int j=0; j<Piece::SIZE; ++j)
      {
	const Piece target = state.getPieceOf(j);
	const Piece black_king = state.getKingPiece<BLACK>();
	const Piece white_king = state.getKingPiece<WHITE>();

	const int attack_black = AttackKing::valueOf(black_king, target);
	const int attack_white = AttackKing::valueOf(white_king, target);
	CPPUNIT_ASSERT((attack_black == 0) || (attack_white == 0));

	const int defense_black = DefenseKing::valueOf(black_king, target);
	const int defense_white = DefenseKing::valueOf(white_king, target);
	CPPUNIT_ASSERT((defense_black == 0) || (defense_white == 0));
	
	const int attack_defense 
	  = AttackDefense::valueOf(black_king, white_king, target);
	CPPUNIT_ASSERT_EQUAL(max(target.owner(), 
				 attack_black  + attack_white,
				 defense_black + defense_white),
			     attack_defense);
      }
    }
  }
}

void EndgameAttackDeffenseTest::testStand() 
{
  const Position king_position(5,5);
  const Piece black_king = Piece::makeKing(BLACK, king_position);
  const Piece white_king = Piece::makeKing(WHITE, king_position);

  const int black_value
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(BLACK,GOLD), 
			     Position::STAND());
  CPPUNIT_ASSERT_EQUAL(static_cast<int>(Ptype_Eval_Table.value(GOLD) * 1.1),
		       black_value);

  const int white_value
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(WHITE,GOLD), 
			     Position::STAND());
  CPPUNIT_ASSERT_EQUAL(static_cast<int>(-Ptype_Eval_Table.value(GOLD) * 1.1),
		       white_value);

  const int black_rook
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(BLACK,ROOK), 
			     Position::STAND());
  CPPUNIT_ASSERT_EQUAL(Ptype_Eval_Table.value(ROOK) +
		       Ptype_Eval_Table.value(PAWN), black_rook);

  const int white_rook
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(WHITE,ROOK), 
			     Position::STAND());
  CPPUNIT_ASSERT_EQUAL(-Ptype_Eval_Table.value(ROOK) -
		       Ptype_Eval_Table.value(PAWN), white_rook);
}

void EndgameAttackDeffenseTest::testConsistentUpdate()
{
  consistencyTestUpdate<AttackDefense>();
}

void EndgameAttackDeffenseTest::testConsistentExpect()
{
  consistencyTestExpect<AttackDefense>();
}

void EndgameAttackDeffenseTest::testYagura()
{
  {
    NumEffectState in_castle(CsaString(
			       "P1-KY-KE *  *  *  *  * -KE-KY\n"
			       "P2 * -HI *  *  *  * -KI-OU * \n"
			       "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
			       "P4-FU * -FU * -FU-FU-FU * -FU\n"
			       "P5 * -FU *  *  *  *  * +FU * \n"
			       "P6+FU * +FU+FU+FU * +FU * +FU\n"
			       "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
			       "P8 * +OU+KI *  *  *  * +HI * \n"
			       "P9+KY+KE *  *  *  *  * +KE+KY\n"
			       "+\n").getInitialState());
    NumEffectState out_castle1(CsaString(
				 "P1-KY-KE *  *  *  *  * -KE-KY\n"
				 "P2 * -HI *  *  *  * -KI-OU * \n"
				 "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
				 "P4-FU * -FU * -FU-FU-FU * -FU\n"
				 "P5 * -FU *  *  *  *  * +FU * \n"
				 "P6+FU * +FU+FU+FU * +FU * +FU\n"
				 "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
				 "P8 *  * +KI *  *  *  * +HI * \n"
				 "P9+KY+KE+OU *  *  *  * +KE+KY\n"
				 "+\n").getInitialState());
    NumEffectState out_castle2(CsaString(
				 "P1-KY-KE *  *  *  *  * -KE-KY\n"
				 "P2 * -HI *  *  *  * -KI-OU * \n"
				 "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
				 "P4-FU * -FU * -FU-FU-FU * -FU\n"
				 "P5 * -FU *  *  *  *  * +FU * \n"
				 "P6+FU * +FU+FU+FU * +FU * +FU\n"
				 "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
				 "P8 *  * +KI *  *  *  * +HI * \n"
				 "P9+KY+KE * +OU *  *  * +KE+KY\n"
				 "+\n").getInitialState());
    NumEffectState out_castle3(CsaString(
				 "P1-KY-KE *  *  *  *  * -KE-KY\n"
				 "P2 * -HI *  *  *  * -KI-OU * \n"
				 "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
				 "P4-FU * -FU * -FU-FU-FU * -FU\n"
				 "P5 * -FU *  *  *  *  * +FU * \n"
				 "P6+FU * +FU+FU+FU * +FU * +FU\n"
				 "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
				 "P8 *  * +KI *  *  *  * +HI * \n"
				 "P9+KY+KE * +OU *  *  * +KE+KY\n"
				 "+\n").getInitialState());
    AttackDefense in(in_castle), out1(out_castle1), out2(out_castle2),
      out3(out_castle3);
    CPPUNIT_ASSERT(in.value(BLACK) > out1.value(BLACK));
    CPPUNIT_ASSERT(out1.value(BLACK) > out2.value(BLACK));
    CPPUNIT_ASSERT(out1.value(BLACK) > out3.value(BLACK));
    CPPUNIT_ASSERT(in.value() - out1.value() > -100); // 本当は左辺大にしたいけど難しい
    CPPUNIT_ASSERT(out1.value() > out2.value());
    CPPUNIT_ASSERT(out1.value() > out3.value());
  }
}

void EndgameAttackDeffenseTest::testBlockingSilver()
{
  {
    NumEffectState castle1(CsaString(
			     "P1-KY-KE *  *  *  *  * -KE-KY\n"
			     "P2 * -HI *  *  *  * -KI-OU * \n"
			     "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
			     "P4-FU * -FU * -FU-FU-FU * -FU\n"
			     "P5 * -FU *  *  *  *  * +FU * \n"
			     "P6+FU * +FU+FU+FU * +FU * +FU\n"
			     "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
			     "P8 *  * +KI *  *  *  * +HI * \n"
			     "P9+KY+KE+OU *  *  *  * +KE+KY\n"
			     "+\n").getInitialState());
    NumEffectState bad_castle(CsaString(
			     "P1-KY-KE *  *  *  *  * -KE-KY\n"
			     "P2 * -HI *  *  *  * -KI-OU * \n"
			     "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
			     "P4-FU * -FU * -FU-FU-FU * -FU\n"
			     "P5 * -FU *  *  *  *  * +FU * \n"
			     "P6+FU * +FU+FU+FU * +FU * +FU\n"
			     "P7 * +FU * +KI+KA+FU+GI *  * \n"
			     "P8 * +GI+KI *  *  *  * +HI * \n"
			     "P9+KY+KE+OU *  *  *  * +KE+KY\n"
			     "+\n").getInitialState());
    AttackDefense normal(castle1), bad(bad_castle);
    CPPUNIT_ASSERT(normal.value(BLACK) > bad.value(BLACK));
    CPPUNIT_ASSERT(normal.value() > bad.value());
  }
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

==> libosl_0.4.0.orig.tar.gz: libosl_0.4.0.orig/test/eval/endgame/attackDefense.t.cc <==
#include "osl/eval/endgame/attackDefense.h"
#include "osl/eval/evalTraits.h"
#include "osl/eval/pieceEval.h"
#include "osl/record/csaString.h"
#include "../consistencyTest.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace osl;
using namespace osl::eval;
using namespace osl::eval::endgame;

class EndgameAttackDeffenseTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(EndgameAttackDeffenseTest);
  CPPUNIT_TEST(testStand);
  CPPUNIT_TEST(testMax);
  CPPUNIT_TEST(testConsistentUpdate);
  CPPUNIT_TEST(testConsistentExpect);
  CPPUNIT_TEST(testYagura);
  CPPUNIT_TEST(testBlockingSilver);
  CPPUNIT_TEST(testInitial);
  CPPUNIT_TEST_SUITE_END();
public:
  void testMax();
  void testStand();
  void testConsistentUpdate();
  void testConsistentExpect();
  void testYagura();
  void testBlockingSilver();
  void testInitial();
};

CPPUNIT_TEST_SUITE_REGISTRATION(EndgameAttackDeffenseTest);

void EndgameAttackDeffenseTest::testInitial()
{
  const NumEffectState state(CsaString(
			       "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			       "P2 * -HI *  *  *  *  * -KA * \n"
			       "P3-FU-FU-FU-FU-FU-FU-FU-FU-FU\n"
			       "P4 *  *  *  *  *  *  *  *  * \n"
			       "P5 *  *  *  *  *  *  *  *  * \n"
			       "P6 *  *  *  *  *  *  *  *  * \n"
			       "P7+FU+FU+FU+FU+FU+FU+FU+FU+FU\n"
			       "P8 * +KA *  *  *  *  * +HI * \n"
			       "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			       "+\n").getInitialState());
  AttackDefense eval(state);
  CPPUNIT_ASSERT_EQUAL(0, eval.value());
}

void EndgameAttackDeffenseTest::testMax() 
{
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  std::string file_name;
  for (int i=0; i<(OslConfig::inUnitTestShort() ? 10 : 100) && (ifs >> file_name) ; i++)
  {
    if ((i % 20) == 0)
      std::cerr << '.';
    if (file_name == "") 
      break;
    file_name = OslConfig::testCsaFile(file_name);

    const Record record=CsaFile(file_name).getRecord();
    const vector<osl::Move> moves=record.getMoves();

    NumEffectState state(record.getInitialState());
    for (unsigned int i=0; i<moves.size(); i++)
    {
      const Move m = moves[i];
      state.makeMove(m);

      for (int j=0; j<Piece::SIZE; ++j)
      {
	const Piece target = state.pieceOf(j);
	const Piece black_king = state.kingPiece<BLACK>();
	const Piece white_king = state.kingPiece<WHITE>();

	const int attack_black = AttackKing::valueOf(black_king, target);
	const int attack_white = AttackKing::valueOf(white_king, target);
	CPPUNIT_ASSERT((attack_black == 0) || (attack_white == 0));

	const int defense_black = DefenseKing::valueOf(black_king, target);
	const int defense_white = DefenseKing::valueOf(white_king, target);
	CPPUNIT_ASSERT((defense_black == 0) || (defense_white == 0));
	
	const int attack_defense 
	  = AttackDefense::valueOf(black_king, white_king, target);
	CPPUNIT_ASSERT_EQUAL(max(target.owner(), 
				 attack_black  + attack_white,
				 defense_black + defense_white),
			     attack_defense);
      }
    }
  }
}

void EndgameAttackDeffenseTest::testStand() 
{
  const Square king_position(5,5);
  const Piece black_king = Piece::makeKing(BLACK, king_position);
  const Piece white_king = Piece::makeKing(WHITE, king_position);

  const int black_value
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(BLACK,GOLD), 
			     Square::STAND());
  CPPUNIT_ASSERT_EQUAL(static_cast<int>(Ptype_Eval_Table.value(GOLD) * 1.1),
		       black_value);

  const int white_value
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(WHITE,GOLD), 
			     Square::STAND());
  CPPUNIT_ASSERT_EQUAL(static_cast<int>(-Ptype_Eval_Table.value(GOLD) * 1.1),
		       white_value);

  const int black_rook
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(BLACK,ROOK), 
			     Square::STAND());
  CPPUNIT_ASSERT_EQUAL(Ptype_Eval_Table.value(ROOK) +
		       Ptype_Eval_Table.value(PAWN), black_rook);

  const int white_rook
    = AttackDefense::valueOf(black_king, white_king, newPtypeO(WHITE,ROOK), 
			     Square::STAND());
  CPPUNIT_ASSERT_EQUAL(-Ptype_Eval_Table.value(ROOK) -
		       Ptype_Eval_Table.value(PAWN), white_rook);
}

void EndgameAttackDeffenseTest::testConsistentUpdate()
{
  consistencyTestUpdate<AttackDefense>();
}

void EndgameAttackDeffenseTest::testConsistentExpect()
{
  consistencyTestExpect<AttackDefense>();
}

void EndgameAttackDeffenseTest::testYagura()
{
  {
    NumEffectState in_castle(CsaString(
			       "P1-KY-KE *  *  *  *  * -KE-KY\n"
			       "P2 * -HI *  *  *  * -KI-OU * \n"
			       "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
			       "P4-FU * -FU * -FU-FU-FU * -FU\n"
			       "P5 * -FU *  *  *  *  * +FU * \n"
			       "P6+FU * +FU+FU+FU * +FU * +FU\n"
			       "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
			       "P8 * +OU+KI *  *  *  * +HI * \n"
			       "P9+KY+KE *  *  *  *  * +KE+KY\n"
			       "+\n").getInitialState());
    NumEffectState out_castle1(CsaString(
				 "P1-KY-KE *  *  *  *  * -KE-KY\n"
				 "P2 * -HI *  *  *  * -KI-OU * \n"
				 "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
				 "P4-FU * -FU * -FU-FU-FU * -FU\n"
				 "P5 * -FU *  *  *  *  * +FU * \n"
				 "P6+FU * +FU+FU+FU * +FU * +FU\n"
				 "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
				 "P8 *  * +KI *  *  *  * +HI * \n"
				 "P9+KY+KE+OU *  *  *  * +KE+KY\n"
				 "+\n").getInitialState());
    NumEffectState out_castle2(CsaString(
				 "P1-KY-KE *  *  *  *  * -KE-KY\n"
				 "P2 * -HI *  *  *  * -KI-OU * \n"
				 "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
				 "P4-FU * -FU * -FU-FU-FU * -FU\n"
				 "P5 * -FU *  *  *  *  * +FU * \n"
				 "P6+FU * +FU+FU+FU * +FU * +FU\n"
				 "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
				 "P8 *  * +KI *  *  *  * +HI * \n"
				 "P9+KY+KE * +OU *  *  * +KE+KY\n"
				 "+\n").getInitialState());
    NumEffectState out_castle3(CsaString(
				 "P1-KY-KE *  *  *  *  * -KE-KY\n"
				 "P2 * -HI *  *  *  * -KI-OU * \n"
				 "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
				 "P4-FU * -FU * -FU-FU-FU * -FU\n"
				 "P5 * -FU *  *  *  *  * +FU * \n"
				 "P6+FU * +FU+FU+FU * +FU * +FU\n"
				 "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
				 "P8 *  * +KI *  *  *  * +HI * \n"
				 "P9+KY+KE * +OU *  *  * +KE+KY\n"
				 "+\n").getInitialState());
    AttackDefense in(in_castle), out1(out_castle1), out2(out_castle2),
      out3(out_castle3);
    CPPUNIT_ASSERT(in.value(BLACK) > out1.value(BLACK));
    CPPUNIT_ASSERT(out1.value(BLACK) > out2.value(BLACK));
    CPPUNIT_ASSERT(out1.value(BLACK) > out3.value(BLACK));
    CPPUNIT_ASSERT(in.value() - out1.value() > -100); // 本当は左辺大にしたいけど難しい
    CPPUNIT_ASSERT(out1.value() > out2.value());
    CPPUNIT_ASSERT(out1.value() > out3.value());
  }
}

void EndgameAttackDeffenseTest::testBlockingSilver()
{
  {
    NumEffectState castle1(CsaString(
			     "P1-KY-KE *  *  *  *  * -KE-KY\n"
			     "P2 * -HI *  *  *  * -KI-OU * \n"
			     "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
			     "P4-FU * -FU * -FU-FU-FU * -FU\n"
			     "P5 * -FU *  *  *  *  * +FU * \n"
			     "P6+FU * +FU+FU+FU * +FU * +FU\n"
			     "P7 * +FU+GI+KI+KA+FU+GI *  * \n"
			     "P8 *  * +KI *  *  *  * +HI * \n"
			     "P9+KY+KE+OU *  *  *  * +KE+KY\n"
			     "+\n").getInitialState());
    NumEffectState bad_castle(CsaString(
			     "P1-KY-KE *  *  *  *  * -KE-KY\n"
			     "P2 * -HI *  *  *  * -KI-OU * \n"
			     "P3 *  * -GI-FU-KA-KI-GI-FU * \n"
			     "P4-FU * -FU * -FU-FU-FU * -FU\n"
			     "P5 * -FU *  *  *  *  * +FU * \n"
			     "P6+FU * +FU+FU+FU * +FU * +FU\n"
			     "P7 * +FU * +KI+KA+FU+GI *  * \n"
			     "P8 * +GI+KI *  *  *  * +HI * \n"
			     "P9+KY+KE+OU *  *  *  * +KE+KY\n"
			     "+\n").getInitialState());
    AttackDefense normal(castle1), bad(bad_castle);
    CPPUNIT_ASSERT(normal.value(BLACK) > bad.value(BLACK));
    CPPUNIT_ASSERT(normal.value() > bad.value());
  }
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
